from termcolor import cprint

class Printer:
  def __init__(self, project):
    self.project = project

  def puts_basic_info(self):
    self.puts_project()
    self.puts_cloud()

  def puts_project(self):
    cprint("PROJECT: " + self.project["name"], "blue")
  
  def puts_cloud(self):
    if "aws" in self.project["cloud"]:
      cloud = "AWS"
    elif "google" in self.project["cloud"]:
      cloud = "GOOGLE"
    
    cprint("CLOUD: " + cloud, "blue")      


import sys
from kinton.configuration                    import Configuration
from kinton.github_downloader                import GithubDownloader
from kinton.cloud                            import Cloud
from kinton.ansible                          import Ansible
from kinton.printer                          import Printer
from kinton.config_files                     import ConfigFiles


def main():
  args = sys.argv[1:]
  if (len(args) == 1) and (args[0]=="init"):
    ConfigFiles.create()
  else:
    Configuration.load()
    execute_ansible(args)

def execute_ansible(args):
  projects = get_projects()    

  for project in projects:
    printer = Printer(project)
    printer.puts_basic_info()
    
    cloud = Cloud(project)
    cloud.configure()

    downloader = GithubDownloader(project["github"])

    ansible = Ansible(project["ansible"], downloader, args)
    ansible.run()

def get_projects():
  projects = []
  for key, data in Configuration.projects.items():
    if ("enabled" not in data) or (data["enabled"] == True):
      data["name"] = key
      projects.append(data)
  return projects

if __name__ == "__main__":
  main()



import os

import yaml


class Configuration:
  kinton = None
  projects = None
  aws_secrets = None

  def load():
    with open("kinton.yml", 'r') as ymlfile:
        Configuration.kinton = yaml.load(ymlfile)
    with open("projects.yml", 'r') as ymlfile:
        Configuration.projects = yaml.load(ymlfile)
    with open("aws_secrets.yml", 'r') as ymlfile:
        Configuration.aws_secrets = yaml.load(ymlfile)           



import os
import shutil
import subprocess
from github import Github
from github import GithubException
from kinton.configuration import Configuration
from kinton.file_system import FileSystem

class GithubDownloader:
  
  def __init__(self, github_config):
    self.github = Github(Configuration.kinton["github_token"])
    organization = self.github.get_organization(github_config["organization"])
    self.repository = organization.get_repo(github_config["respository"])

    self.tmp_dir = Configuration.kinton["defaults"]["tmp_dir"]
  
  def get_folder(self, folder_path, sha="master", exclude_dirs=[]):
    path = self.tmp_dir + folder_path
    FileSystem.create_folder(path)
    self.download_folder(folder_path, sha, exclude_dirs)

  def download_folder(self, folder_path, sha ,exclude_dirs):
    contents = self.repository.get_dir_contents(folder_path, ref=sha)
    
    for content in contents:
      if content.type == 'dir':
        folder_name = content.path.split("/")[-1]
        if folder_name not in exclude_dirs:
          self.create_folder(content.path)
          self.get_folder(content.path, sha, exclude_dirs)
      else:
        try:
          path = content.path
          file_content = self.repository.get_contents(path, ref=sha)
          file_data = file_content.decoded_content.decode('utf-8')
          output_path = self.tmp_dir + content.path
          file_out = open(output_path, "w")
          file_out.write(file_data)
          file_out.close()
        except (GithubException, IOError) as exc:
          print('Error processing %s: %s', content.path, exc)

  def create_folder(self, folder=""):
    path = self.tmp_dir + folder
    FileSystem.create_folder(path)


import os
import re
import sys
from jinja2 import Environment
from jinja2 import FileSystemLoader
from pathlib import Path
THIS_DIR = os.path.dirname(os.path.abspath(__file__))

class ConfigFiles:
  def create():
    files = ["kinton.yml", "projects.yml", "aws_secrets.yml"]
    for filename in files:   
      the_file = Path(filename)
      if the_file.exists():
        ConfigFiles.request_operation(filename)
      else:
        ConfigFiles.write_file(filename)

  def request_operation(filename):
    msg = "The file {:s} exists, do you want rewrite it? (y) or (n) ".format(filename)
    answer = input(msg)
    if answer == "y":
      ConfigFiles.write_file(filename)
    elif answer == "n":
      return
    else:
      ConfigFiles.request_operation(filename)
  
  def write_file(filename):    
    j2_env = Environment(loader=FileSystemLoader(THIS_DIR +'/templates'), trim_blocks=True)
    template = j2_env.get_template(filename + ".j2") 
    rendered_file = template.render()
    file_out = open("./" + filename, "w")
    file_out.write(rendered_file)
    file_out.close()    




import os
import datetime
from jinja2 import Environment
from jinja2 import FileSystemLoader

THIS_DIR = os.path.dirname(os.path.abspath(__file__))

class Aws:
  def switch_account(aws_config):
    j2_env = Environment(loader=FileSystemLoader(THIS_DIR +'/templates'), trim_blocks=True)
    template = j2_env.get_template('aws_credentials.j2') 

    rendered_file = template.render(aws_access_key_id=aws_config["aws_access_key_id"], 
                                    aws_secret_access_key=aws_config["aws_secret_access_key"],
                                    region=aws_config["region"],
                                    now=datetime.datetime.utcnow()) 
    file_out = open("/Users/jesus.sayar/.aws/credentials", "w")
    file_out.write(rendered_file)
    file_out.close()


import subprocess
import datetime
from kinton.aws import Aws
from kinton.google import Google
from kinton.configuration import Configuration

class Cloud:
  def __init__(self, project_config):
    self.project_config = project_config

  def configure(self):
    if "aws" in self.project_config["cloud"]:
      aws_config = Configuration.aws_secrets[self.project_config["name"]]
      Aws.switch_account(aws_config)
    elif "google" in self.cloud_config:
      google_config = self.project_config["cloud"]["google"]
      Google.switch_account(google_config)


import os
import subprocess
from termcolor import cprint
from kinton.configuration import Configuration
from kinton.cloud import Cloud
from kinton.file_system import FileSystem


class Ansible:
  ANSIBLE_DIR = '/ansible/'
  PLAYBOOK_FILE = "playbook.yml"
  SCRIPT = "/bin/ansible.sh"

  THIS_DIR = os.path.dirname(os.path.abspath(__file__))  

  def __init__(self, ansible_config, downloader, cmd_args):
    self.ansible_config = ansible_config
    self.downloader = downloader
    self.cmd_args = self.parse_args(cmd_args)

    self.tmp_dir = Configuration.kinton["defaults"]["tmp_dir"]
    self.settings = Configuration.kinton["defaults"]["ansible"]

  def run(self):
    self.create_temp_folder()
    self.download_temp_files()
    self.execute_command()
    self.remove_temp_folder()

  def create_temp_folder(self):
    FileSystem.create_folder(self.tmp_dir)

  def download_temp_files(self):
    exclude_dirs = self.settings["exclude_dirs"]
    if "exclude_dirs" in self.ansible_config:
      exclude_dirs += self.ansible_config["exclude_dirs"]
    self.downloader.get_folder(self.ansible_config["ansible_dir"], exclude_dirs=exclude_dirs)
  
  def execute_command(self):
    for inventory in self.ansible_config["inventories"]:
      ansible_dir = self.tmp_dir + self.ansible_config['ansible_dir']
      script_path = self.THIS_DIR + self.SCRIPT
      command = ["/bin/bash", script_path, ansible_dir, inventory]
    
      for arg in self.cmd_args:
        command.append(arg)

      command.append("-u")
      command.append(self.ansible_config["remote_user"])
            
      subprocess.run(command)

  def remove_temp_folder(self):
    FileSystem.remove_folder(self.tmp_dir)

  def parse_args(self, cmd_args):
    for index, item in enumerate(cmd_args):
      if item.endswith(".yml"):
        current_path = os.getcwd()
        cmd_args[index] = current_path +  "/" + item
    return cmd_args



from setuptools import setup

setup(name='kinton',
      version='0.1',
      description='Kinton',
      keywords='ansible infrastructure provisioning',
      url='https://github.com/thecocktail/kinton',
      author='Jesus Sayar',
      author_email='jesus.sayar@the-cocktail.com',
      license='MIT',
      packages=['kinton'],
      install_requires=[
          'PyYAML==3.12',
          'ipdb==0.10.3',
          'termcolor==1.1.0',
          'pygithub==1.35',
          'Jinja2==2.9.6'
      ],
      entry_points = {
        'console_scripts': ['kinton=kinton.command_line:main'],
      },      
      include_package_data=True,
      zip_safe=False)
